################################################################################
####### This R code generates Figure 1, Table 4 and Table 5 in the paper #######
################################################################################

source("./core.R")
library(QualInt) # for IBGA and LRT tests

#### Writing-to-learn
library(metafor)
dat <- get(data(dat.bangertdrowns2004))

#### Right panel of Figure 1
res <- rma(yi, vi, data=dat)
## pdf("bangertdrowns_2004_forest.pdf", 8, 12)
forest(res, slab = paste0(dat$author, " (", dat$year, ")"), order = "obs")
## dev.off()

p.values <- as.vector(pnorm(dat$yi / sqrt(dat$vi)))
p1 <- test.global.null(p.values, method = "Fisher")
p2 <- test.global.null(p.values, method = "Fisher", trunc = 0.8)
p3 <- test.global.null(p.values, method = "Fisher", trunc = 0.5)
p32 <- test.global.null(p.values, method = "Fisher", trunc = find.trunc(p.values, alpha = 0.5))
p4 <- test.global.null(p.values, method = "Bonferroni")
p5 <- test.global.null(p.values, method = "Bonferroni", trunc = 0.8)
p6 <- test.global.null(p.values, method = "Bonferroni", trunc = 0.5)
p62 <- test.global.null(p.values, method = "Bonferroni", trunc = find.trunc(p.values, alpha = 0.05))

p.pos <- c(p1, p2, p3, p32, p4, p5, p6, p62)

p.values <- as.vector(pnorm(- dat$yi / sqrt(dat$vi)))
tau.adaptive <- find.trunc(p.values)
p1 <- test.global.null(p.values, method = "Fisher")
p2 <- test.global.null(p.values, method = "Fisher", trunc = 0.8)
p3 <- test.global.null(p.values, method = "Fisher", trunc = 0.5)
p32 <- test.global.null(p.values, method = "Fisher", trunc = find.trunc(p.values, alpha = 0.5))
p4 <- test.global.null(p.values, method = "Bonferroni")
p5 <- test.global.null(p.values, method = "Bonferroni", trunc = 0.8)
p6 <- test.global.null(p.values, method = "Bonferroni", trunc = 0.5)
p62 <- test.global.null(p.values, method = "Bonferroni", trunc = find.trunc(p.values, alpha = 0.05))

p.neg <- c(p1, p2, p3, p32, p4, p5, p6, p62)

#### Third cell of Table 5
apply(rbind(p.pos, p.neg), 2, max)

p.ibga <- qualval(dat$yi, sqrt(dat$vi), "IBGA")$pvalue
p.lrt <- qualval(dat$yi, sqrt(dat$vi), "LRT")$pvalue
c(p.ibga, p.lrt)



#### Modified school calendar
library(metafor)
dat <- get(data(dat.konstantopoulos2011))

res <- rma(yi, vi, data=dat)
print(res, digits=3)

res.ml <- rma.mv(yi, vi, random = ~ 1 | district/school, data=dat)

#### Left panel of Figure 1
## pdf("konstantopoulos_2011_forest.pdf", 8, 12)
forest(res.ml, slab = paste0("Study: ", dat$study, ", District: ", dat$district), order = "obs")
## dev.off()


p.values <- as.vector(pnorm(dat$yi / sqrt(dat$vi)))
p1 <- test.global.null(p.values, method = "Fisher")
p2 <- test.global.null(p.values, method = "Fisher", trunc = 0.8)
p3 <- test.global.null(p.values, method = "Fisher", trunc = 0.5)
p32 <- test.global.null(p.values, method = "Fisher", trunc = find.trunc(p.values, alpha = 0.5))
p4 <- test.global.null(p.values, method = "Bonferroni")
p5 <- test.global.null(p.values, method = "Bonferroni", trunc = 0.8)
p6 <- test.global.null(p.values, method = "Bonferroni", trunc = 0.5)
p62 <- test.global.null(p.values, method = "Bonferroni", trunc = find.trunc(p.values, alpha = 0.05))

p.pos <- c(p1, p2, p3, p32, p4, p5, p6, p62)

p.values <- as.vector(pnorm(- dat$yi / sqrt(dat$vi)))
p1 <- test.global.null(p.values, method = "Fisher")
p2 <- test.global.null(p.values, method = "Fisher", trunc = 0.8)
p3 <- test.global.null(p.values, method = "Fisher", trunc = 0.5)
p32 <- test.global.null(p.values, method = "Fisher", trunc = find.trunc(p.values, alpha = 0.5))
p4 <- test.global.null(p.values, method = "Bonferroni")
p5 <- test.global.null(p.values, method = "Bonferroni", trunc = 0.8)
p6 <- test.global.null(p.values, method = "Bonferroni", trunc = 0.5)
p62 <- test.global.null(p.values, method = "Bonferroni", trunc = find.trunc(p.values, alpha = 0.05))

p.neg <- c(p1, p2, p3, p32, p4, p5, p6, p62)

#### First cell of Table 5
apply(rbind(p.pos, p.neg), 2, max)

p.ibga <- qualval(dat$yi, sqrt(dat$vi), "IBGA")$pvalue
p.lrt <- qualval(dat$yi, sqrt(dat$vi), "LRT")$pvalue
c(p.ibga, p.lrt)


#### Table 4
dat$district <- factor(dat$district)
dat$study <- factor(dat$study)

## Random effects model
x <- rma.mv(yi, vi, random = ~ 1 | district/school, data=dat)
table.overall <-
    cbind(estimate = c(x$b), se = x$se, zval = x$zval,
          pval = x$pval, ci.lb = x$ci.lb, ci.ub = x$ci.ub)
rownames(table.overall) <- "Overall"

## Fixed effects (for district)
x <- rma.mv(yi ~ district - 1, vi, random = ~ 1 | school, data=dat)
table.district <-
    cbind(estimate = c(x$b), se = x$se, zval = x$zval,
          pval = x$pval, ci.lb = x$ci.lb, ci.ub = x$ci.ub)

rownames(table.district) <- paste("District", levels(dat$district))

table <- rbind(table.overall, table.district)

library(xtable)
print(xtable(table[, 1:4], digits = c(0, 3, 3, 2, 3)))



p.values <- pnorm(x$zval)

p1 <- test.global.null(p.values, method = "Fisher")
p2 <- test.global.null(p.values, method = "Fisher", trunc = 0.8)
p3 <- test.global.null(p.values, method = "Fisher", trunc = 0.5)
p32 <- test.global.null(p.values, method = "Fisher", trunc = find.trunc(p.values, alpha = 0.5))
p4 <- test.global.null(p.values, method = "Bonferroni")
p5 <- test.global.null(p.values, method = "Bonferroni", trunc = 0.8)
p6 <- test.global.null(p.values, method = "Bonferroni", trunc = 0.5)
p62 <- test.global.null(p.values, method = "Bonferroni", trunc = find.trunc(p.values, alpha = 0.05))

p.pos <- c(p1, p2, p3, p32, p4, p5, p6, p62)

p.values <- 1 - pnorm(x$zval)
p1 <- test.global.null(p.values, method = "Fisher")
p2 <- test.global.null(p.values, method = "Fisher", trunc = 0.8)
p3 <- test.global.null(p.values, method = "Fisher", trunc = 0.5)
p32 <- test.global.null(p.values, method = "Fisher", trunc = find.trunc(p.values, alpha = 0.5))
p4 <- test.global.null(p.values, method = "Bonferroni")
p5 <- test.global.null(p.values, method = "Bonferroni", trunc = 0.8)
p6 <- test.global.null(p.values, method = "Bonferroni", trunc = 0.5)
p62 <- test.global.null(p.values, method = "Bonferroni", trunc = find.trunc(p.values, alpha = 0.05))

p.neg <- c(p1, p2, p3, p32, p4, p5, p6, p62)


#### Second cell of Table 5
apply(rbind(p.pos, p.neg), 2, max)

p.ibga <- qualval(x$zval, rep(1, length(x$zval)), "IBGA")$pvalue
p.lrt <- qualval(x$zval, rep(1, length(x$zval)), "LRT")$pvalue
c(p.ibga, p.lrt)
